@@ -1,5 +1,5 @@
-/* The C part is broken into three pieces, "Json3-common.c",
-   "Json3-perl.c", and "Json3-entry-points.c". This file contains the
+/* The C part is broken into three pieces, "json-common.c",
+   "json-perl.c", and "json-entry-points.c". This file contains the
    "Perl" stuff, for example if we have a string, the stuff to convert
    it into a Perl hash key or a Perl scalar is in this file. */
 
@@ -281,7 +281,7 @@ string_number_end:
        digits. */
 
 #ifdef PERLING
-    return newSVpv (start, (char *) parser->end - start);
+	return newSVpv (start, (STRLEN) ((char *) parser->end - start));
 #elif defined (TOKENING)
     return json_token_new (parser, (unsigned char *) start,
 			   parser->end - 1, json_token_number);
@@ -296,10 +296,14 @@ PREFIX(string) (parser_t * parser)
     unsigned char c;
 #ifdef PERLING
     SV * string;
+    STRLEN len;
 #elif defined (TOKENING)
     json_token_t * string;
-#endif
     int len;
+#else
+    int len;
+#endif
+
     unsigned char * start;
 
     start = parser->end;
@@ -414,7 +418,6 @@ PREFIX(literal_true) (parser_t * parser)
 	FAILLITERAL('u');
     }
     FAILLITERAL('r');
-
 }
 
 static SVPTR
@@ -443,7 +446,6 @@ PREFIX(literal_false) (parser_t * parser)
 	FAILLITERAL('l');
     }
     FAILLITERAL('a');
-
 }
 
 static SVPTR
@@ -529,7 +531,7 @@ PREFIX(array) (parser_t * parser)
     unsigned char * start;
 #ifdef PERLING
     AV * av;
-    SV * value;
+    SV * value = & PL_sv_undef;
 #elif defined (TOKENING)
     json_token_t * av;
     json_token_t * prev;
@@ -561,7 +563,7 @@ PREFIX(array) (parser_t * parser)
 #ifdef PERLING
     av_push (av, value);
 #elif defined (TOKENING)
-    prev = json_token_set_child (av, value);
+    prev = json_token_set_child (parser, av, value);
 #endif
 
     /* Accept either a comma or whitespace or the end of the array. */
@@ -684,7 +686,7 @@ PREFIX(object) (parser_t * parser)
 				json_token_string);
 	/* We only come past the label "hash_start" once, so we don't
 	   need to check that there is not already a child. */
-	json_token_set_child (hv, value);
+	json_token_set_child (parser, hv, value);
 	prev = value;
 #endif
 	get_key_string (parser, & key);
